/* Run a CMS command. */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include "lib3270.h"

static void waitread(l3h_t);
static int cmd_done(l3h_t);
static void cms_cmd(l3h_t, const char *cmd);

static int verbose = 0;

int
main(int argc, char *argv[])
{
	static const char *opts[] = { /*"-trace",*/ NULL };
	char *host;
	char *user;
	char *pass;
	char *cmd;
	l3h_t h;

	if (argc != 5) {
		fprintf(stderr, "usage: %s host user pass cmd\n", argv[0]);
		exit(1);
	}
	host = argv[1];
	user = argv[2];
	pass = argv[3];
	cmd = argv[4];

	/* Init the library. */
	if (open_3270(0L, opts, &h) < 0) {
		fprintf(stderr, "open failed\n");
		exit(1);
	}
	if (verbose)
		printf("library initialized\n");

	/* Connect to the host. */
	if (aConnect(h, host) < 0) {
		fprintf(stderr, "Connect failed: %s\n", result_3270(h, NULL));
		exit(1);
	}
	if (verbose)
		printf("connected\n");
	if (aWaitInputField(h, 0) < 0) {
		fprintf(stderr, "WaitInputField failed: %s\n",
		    result_3270(h, NULL));
		exit(1);
	}

	/* Log in. */
	aString(h, user);
	sTab(h);
	aString(h, pass);
	aEnter(h);
	waitread(h);

	/* Check for existing login. */
	if (!strcmp(sAsciiLine(h, 1, 11, 7), "Already")) {
		fprintf(stderr, "Can't run -- already logged in\n");
		exit(1);
	}

	/*
	 * If we're in CP mode, which means we disconnected last time, boot
	 * CMS.
	 */
	if (!strcmp(sAsciiLine(h, sRows(h) - 1, sCols(h) - 20, 2), "CP")) {
		if (verbose)
			printf("booting CMS\n");
		aClear(h);
		aString(h, "i cms");
		aEnter(h);
		waitread(h);
	}

	/*
	 * Enter an empty command to get a CMS prompt.  If we don't do this,
	 * there will be a Ready prompt as the first line of output below.
	 */
	aClear(h);
	aEnter(h);
	(void) cmd_done(h);

	/* Enter the user's command. */
	cms_cmd(h, cmd);

	/* Log off. */
	aClear(h);
	if (verbose)
		printf("logging off\n");
	aString(h, "logoff");
	aEnter(h);
	aWaitDisconnect(h, 0);

	return 0;
}

static void
waitread(l3h_t h)
{
	const char *s;

	sSnapSave(h);
	while (strcmp((s = sSnapAsciiLine(h, sSnapRows(h) - 1,
			sSnapCols(h) - 17, 4)),
			"READ")) {
		if (verbose)
			printf("not a read (%s)\n", s);
		aSnapWaitOutput(h, 0);
	}
}

static int
all_spaces(char *s)
{
	char c;

	while ((c = *s++)) {
		if (c != ' ')
			return 0;
	}
	return 1;
}

static int
cmd_done(l3h_t h)
{
	int i;
	char text[133];

	sSnapSave(h);
	for (;;) {
		if (!strcmp(sSnapAsciiLine(h, sSnapRows(h) - 1,
					   sSnapCols(h) - 20, 7),
			    "MORE...")) {
			if (verbose)
				printf("cmd_done: MORE\n");
			return -1;
		}
		i = sSnapRows(h) - 2;
		while (i >= 0) {
			strcpy(text, sSnapAsciiLine(h, i, 0, sSnapCols(h)));
			if (!strncmp(text, "Ready; T=", 9)) {
				if (verbose)
					printf("cmd_done: Ready (success) on "
					    "row %d\n", i);
				return i;
			} else if (!strncmp(text, "Ready(", 6)) {
				fprintf(stderr, "error: %s\n",
				    sSnapAsciiLine(h, i - 1, 0, sSnapCols(h)));
				exit(1);
			} else if (all_spaces(text)) {
				if (verbose)
					printf("cmd_done: row %d all spaces\n",
					    i);
			} else {
				if (verbose)
					printf("cmd_done: not all spaces\n");
				i = 0;
			}
			i--;
		}
		aSnapWaitOutput(h, 0);
	}
}

static void
cms_cmd(l3h_t h, const char *cmd)
{
	int first = 1;
	char text[133];

	/* Clear the screen and enter the command. */
	aClear(h);
	aString(h, cmd);
	aEnter(h);

	for (;;) {
		int d;
		int i;
		int last;

		d = cmd_done(h);
		i = first;
		first = 0;
		if (d < 0) {
			last = sSnapRows(h) - 2;
		} else {
			last = d;
		}
		while (i < last) {
			int sl;

			(void) strcpy(text, sSnapAsciiLine(h, i, 0,
							   sSnapCols(h)));

			if ((sl = strlen(text))) {
				char *s = text + sl - 1;

				while (s > text && *s == ' ') {
					*s-- = '\0';
				}
			}

			printf("%s\n", text);
			i = i + 1;
		}
		if (d >= 0)
			break;

		aClear(h);
	}
}
